AbsorbTracker = LibStub("AceAddon-3.0"):NewAddon("AbsorbTracker", "AceConsole-3.0", "AceEvent-3.0")
AbsorbTrackerGUI = LibStub("AceGUI-3.0")
local media = LibStub("LibSharedMedia-3.0")
local db
local valgbtl = 0
local destgbtl
local playername = UnitName("player")
local spellIds = {
[17] = true, 	-- Power Word: Shield
[47753] = true, -- Divine Aegis
[86273] = true, -- Illuminated Healing
[77535] = true, -- Blood Shield
[96263] = true, -- Sacred Shield
[88063] = true, -- Guarded by the Light
[11426] = true, -- Ice barrier
}

local defaults = {
	profile = {
		font = "Friz Quadrata TT",
		fontscale = 12,
		texture = "Blizzard",
		xpos = 0,
		ypos = 0,
		tankmode = false,
	},
}

local options = {
	type = "group",
	childGroups = "tab",
	args = {
		settings = {
		type = "group",
		name = "Settings",
		args = {
				fontsettingsdesc = {
					order = 1,
					type = "description",
					name = "Font settings\n",
					cmdHidden = true
				},
				font = {
					order = 2,
					type = "select",
					name = "Font",
					desc = "Set the font.",
					values = media:HashTable("font"),
					dialogControl = "LSM30_Font",
					get = function(info) return db.font end,
					set = function(info, f) 
					db.font = f
					AbsorbTracker:frame()
					end,
				},
				fontscale = {
					order = 3,
					name = "Font Scale",
					desc = "Set the font scale",
					type = 'range',
					min = 1,
					max = 50,
					step = 1,
					get = function(info) return db.fontscale end,
					set = function(info, fontsc) 
					db.fontscale = fontsc
					AbsorbTracker:frame()
					end,
				},
				outline = {
					order = 4,
					name = "Outline",
					desc = "Outline",
					type = 'select',
					values = {["OUTLINE"]="Outline",["THICKOUTLINE"]="Thickoutline",["MONOCHROME"]="Monochrome",["MONOCHROME,OUTLINE"]="Monochrome + Outline",["MONOCHROME,THICKOUTLINE"]="Monochrome + Thickoutline",["NONE"]="None"},
					get = function(info) return db.outline end,
					set = function(info, line)
					db.outline = line
					AbsorbTracker:frame()
					end,
				},
				posdesc = {
					order = 5,
					type = "description",
					name = "Position \n",
					cmdHidden = true
				},
				xpos = {
					order = 7,
					name = "X",
					desc = "Change horizontal position",
					type = 'range',
					min = -500,
					max = 500,
					step = 1,
					get = function(info) return db.xpos end,
					set = function(info, x) 
					db.xpos = x
					AbsorbTracker:frame()
					end,
				},
				ypos = {
					order = 10,
					name = "Y",
					desc = "Change veritcal position",
					type = 'range',
					min = -500,
					max = 500,
					step = 1,
					get = function(info) return db.ypos end,
					set = function(info, y) 
					db.ypos = y
					AbsorbTracker:frame()
					end,
				},
				nudgeright = {
					order = 8,
					name = "Right",
					desc = "Nudge right once",
					type = 'execute',
					get = function(info) return db.xpos end,
					func = function() 
					db.xpos = db.xpos + 1
					AbsorbTracker:frame()
					end,
				},
				nudgeleft = {
					order = 6,
					name = "Left",
					desc = "Nudge left once",
					type = 'execute',
					get = function(info) return db.xpos end,
					func = function() 
					db.xpos = db.xpos - 1
					AbsorbTracker:frame()
					end,
				},
				nudgeup = {
					order = 9,
					name = "Up",
					desc = "Nudge up once",
					type = 'execute',
					get = function(info) return db.ypos end,
					func = function() 
					db.ypos = db.ypos + 1
					AbsorbTracker:frame()
					end,
				},
				nudgedown = {
					order = 11,
					name = "Down",
					desc = "Nudge down once",
					type = 'execute',
					get = function(info) return db.ypos end,
					func = function() 
					db.ypos = db.ypos - 1
					AbsorbTracker:frame()
					end,
				},
			},
		},
		extras = {
		type = "group",
		name = "Extras",
			args = {
				tankmode = {
					order = 1,
					name = "Enable tank mode",
					desc = "WARNING! The addon will only track absorbs on your self! This is intended for tanks to use.",
					type = 'toggle',
					get = function(info) return db.tankmode end,
					set = function(info, x) 
					db.tankmode = x
					AbsorbTracker:frame()
					if db.tankmode == true then
						AbsorbTracker:Print("Tank mode is on, only tracking yourself!")
					end
					if db.tankmode == false then
						AbsorbTracker:Print("Tank mode disabled.")
					end
					end,
				},
				posdesc = {
					order = 5,
					type = "description",
					name = "This will make the addon only track absorbs on yourself!\n",
					cmdHidden = true
				},
			},
		},
	},
}

function AbsorbTracker:OnInitialize()
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("AbsorbTracker", "AbsorbTracker")
	
	LibStub("AceConfig-3.0"):RegisterOptionsTable("AbsorbTracker", options, {"AbsorbTracker", "at"})
	LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable("AbsorbTracker", options)
	
	self.db = LibStub("AceDB-3.0"):New("AbsorbTrackerDB", defaults)
	db = self.db.profile
	
	options.args.profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db)
	
	self:RegisterChatCommand("at", "ChatCommand")
	self:RegisterChatCommand("AbsorbTracker", "ChatCommand")
	
	self.db.RegisterCallback(self, "OnProfileChanged", "RefreshConfig")
	self.db.RegisterCallback(self, "OnProfileCopied", "RefreshConfig")
	self.db.RegisterCallback(self, "OnProfileReset", "RefreshConfig")
end
	
function AbsorbTracker:OnEnable()
	self:Print("Ingela's Absorb Tracker loaded. To configurate: /AbsorbTracker or /at. Enjoy!")
	
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	self:RegisterEvent("UNIT_TARGET")
	self:RegisterEvent("SPELL_AURA_REFRESH")
	self:RegisterEvent("SPELL_AURA_REMOVED")
	self:RegisterEvent("SPELL_AURA_APPLIED")
	
	if not media:Fetch("statusbar", db.texture, true) then db.texture = "Blizzard" end
	if not media:Fetch("font", db.font, true) then db.font = "Friz Quadrata TT" end
	
	frame1 = CreateFrame("MessageFrame","frame1",UIParent)
	text = frame1:CreateFontString("test")
	
	CreateFrame( "GameTooltip", "text2", nil, "GameTooltipTemplate" )
	text2:SetOwner( WorldFrame, "ANCHOR_NONE" )
	text2:AddFontStrings(
	text2:CreateFontString( "$parentTextLeft1", nil, "GameTooltipText" ),
	text2:CreateFontString( "$parentTextRight1", nil, "GameTooltipText" ) )
	
	AbsorbTracker:frame()
end

function AbsorbTracker:RefreshConfig()
	db = self.db.profile
end

function AbsorbTracker:ChatCommand(input)
	if not input or input:trim() == "" then
		InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)
	else
		LibStub("AceConfigCmd-3.0").HandleCommand(AbsorbTracker, "at", "AbsorbTracker", input)
	end
end

function AbsorbTracker:frame()
	frame1:SetWidth(300)
	frame1:SetHeight(150)
	frame1:SetFont(media:Fetch("font", db.font), db.fontscale, db.outline~="NONE" and db.outline or nil)
	frame1:SetPoint("CENTER", UIParent, "CENTER", db.xpos, db.ypos)
	frame1:Show()
	
	text:SetFont(media:Fetch("font", db.font), db.fontscale, db.outline~="NONE" and db.outline or nil)
	text:SetPoint("CENTER", UIParent, "CENTER", db.xpos, db.ypos)
	text:SetSize(300, 150)
	text:SetText("0")
	text:Show()
end

function AbsorbTracker:SPELL_AURA_APPLIED(timestamp, event, sourceGUID, sourceName, sourceFlags, destGUID, destName, destFlags, spellId, spellName, spellSchool, auraType, amount)
	if spellIds[spellId] then
		if spellId == 88063 then namegbtl = sourceName valgbtl = amount else valgbtl = 0 end
		AbsorbTracker:runtarget()
	end
end

function AbsorbTracker:SPELL_AURA_REMOVED(timestamp, event, sourceGUID, sourceName, sourceFlags, destGUID, destName, destFlags, spellId, spellName, spellSchool, auraType, amount)
	if spellIds[spellId] then
		if spellId == 88063 then namegbtl = sourceName valgbtl = 0 else valgbtl = 0 end
		AbsorbTracker:runtarget()
	end
end

function AbsorbTracker:SPELL_AURA_REFRESH(timestamp, event, sourceGUID, sourceName, sourceFlags, destGUID, destName, destFlags, spellId, spellName, spellSchool, auraType, amount)
	if spellIds[spellId] then
		if spellId == 88063 then namegbtl = sourceName valgbtl = amount else valgbtl = 0 end
		AbsorbTracker:runtarget()
	end
end

function AbsorbTracker:COMBAT_LOG_EVENT_UNFILTERED(_, timestamp, event, sourceGUID, sourceName, sourceFlags, destGUID, destName, destFlags, amount, spellID, ...)
    if destGUID ~= UnitGUID(db.tankmode and 'player' or 'target') then return end
    local func = self[event]
    if (func) then
        func(self, timestamp, event, sourceGUID, sourceName, sourceFlags, destGUID, destName, destFlags, amount, spellID, ...)
    end
end

function AbsorbTracker:UNIT_TARGET(_, arg1, ...)
	if arg1 == "player" then
		AbsorbTracker:runtarget()
	end
end

function AbsorbTracker:runtarget()
	if db.tankmode == false then
		if UnitIsPlayer("target") and UnitIsFriend("target", "player") and not UnitIsDeadOrGhost("target") then
			frame1:Show()
			text:Show()
			sum = 0
			for t = 1,40 do
			local a,b,c,d,e,f,g,h,i,j,k = UnitBuff("target", t)
				if spellIds[k] then
					if k == 88063 then
					local playertarget = UnitName("target")
					if h == "player" and playertarget == playername then sum = sum + valgbtl end
					if h == "target" and playertarget == namegbtl then sum = sum + valgbtl end
					else
					text2:ClearLines()
					text2:SetUnitBuff("target", t)
					local txt = text2TextLeft2:GetText()
					local _, val, _ = strsplit(" ", txt)
					sum = sum + val
					end
				end
			end
			text:SetText(sum)
		else
			text:Hide()
			frame1:Hide()
		end
	end
	if db.tankmode == true then
		frame1:Show()
		text:Show()
		sum = 0
		for t = 1,40 do
		local a,b,c,d,e,f,g,h,i,j,k = UnitBuff("player", t)
			if spellIds[k] then
				if k == 88063 then
				if h == "player" and namegbtl == playername then sum = sum + valgbtl end
				else
				text2:ClearLines()
				text2:SetUnitBuff("player", t)
				local txt = text2TextLeft2:GetText()
				local _, val, _ = strsplit(" ", txt)
				sum = sum + val
				end
			end
		end
		text:SetText(sum)
	end
end